package com.sun.star.smarttags;

import com.sun.star.awt.XControl;
import com.sun.star.awt.XControlContainer;
import com.sun.star.awt.XControlModel;
import com.sun.star.awt.XDialog;
import com.sun.star.awt.XDialogProvider2;
import com.sun.star.beans.PropertyValue;
import com.sun.star.beans.XPropertySet;
import com.sun.star.container.ElementExistException;
import com.sun.star.container.XStringKeyMap;
import com.sun.star.i18n.Boundary;
import com.sun.star.i18n.WordType;
import com.sun.star.lang.WrappedTargetException;
import com.sun.star.lang.XMultiComponentFactory;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.lang.XSingleComponentFactory;
import com.sun.star.lib.uno.helper.Factory;
import com.sun.star.lib.uno.helper.WeakBase;
import com.sun.star.registry.XRegistryKey;
import com.sun.star.text.TextMarkupType;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;
import com.sun.star.uno.XInterface;
import com.sun.star.util.XChangesBatch;
import com.sun.star.util.XMacroExpander;
import java.util.HashMap;

public final class OOoNameRecognizer extends WeakBase
   implements com.sun.star.lang.XServiceInfo,
              com.sun.star.smarttags.XSmartTagRecognizer,
              com.sun.star.awt.XDialogEventHandler
{
    private final XComponentContext m_xContext;
    private XDialog m_xDialog;
    private XMultiServiceFactory m_xConfProv;
    private XPropertySet m_xConfPropSet;
    private HashMap m_aMap;
    private String m_aDialogFile;
    private static final String m_SmartTagType = "mynamespace#oooname";
    private static final String m_implementationName = OOoNameRecognizer.class.getName();
    private static final String[] m_serviceNames = {
        "com.sun.star.smarttags.SmartTagRecognizer" };


    public OOoNameRecognizer( XComponentContext context ) {
        m_xContext = context;
        m_xDialog = null;
        m_xConfProv = null;
        m_xConfPropSet = null;
        m_aMap = new HashMap();
        
        // get configuration access:
	PropertyValue aPathArgument = new PropertyValue();
	aPathArgument.Name = "nodepath";
	aPathArgument.Value = "/org.openoffice.Office.OOoNameRecognizer/Settings";
        
        Object[] aArgs = new Object[1];
        aArgs[0] = aPathArgument;

        XMultiComponentFactory xFact = m_xContext.getServiceManager();
        Object xObject = null;
        try {
            xObject = xFact.createInstanceWithContext("com.sun.star.configuration.ConfigurationProvider", m_xContext);
        } catch ( com.sun.star.uno.Exception ex ) {
            ex.printStackTrace();
        }
        
        if ( xObject == null )
            return;
        
        m_xConfProv = (XMultiServiceFactory)UnoRuntime.queryInterface(XMultiServiceFactory.class, xObject );

        // try to get read access to configuration:
        try {
            xObject = m_xConfProv.createInstanceWithArguments( "com.sun.star.configuration.ConfigurationUpdateAccess", aArgs );
	} catch ( com.sun.star.uno.Exception ex ) {
            ex.printStackTrace();
	}
        
        if ( xObject == null )
            return;
        
        m_xConfPropSet = (XPropertySet)UnoRuntime.queryInterface(XPropertySet.class, xObject);
        
        try {
            // read map data from configuration:
            final String aMapData = (String) m_xConfPropSet.getPropertyValue( "MapData" );
            FillMapData( aMapData );
            
            String aDialogPath = (String) m_xConfPropSet.getPropertyValue("DialogPath");
            aDialogPath = aDialogPath.substring(20);
            
            // get dialog file patch from configuration
            xObject = m_xContext.getValueByName("/singletons/com.sun.star.util.theMacroExpander");
            
            if ( xObject == null )
                return;
            
            // read dialog file path from configuration:
            final XMacroExpander xMacroExpander = (XMacroExpander)UnoRuntime.queryInterface(XMacroExpander.class, xObject);
            aDialogPath = xMacroExpander.expandMacros(aDialogPath);
            m_aDialogFile = aDialogPath + "/OOoNameRecognizerDialog.xdl";
        } catch ( com.sun.star.uno.Exception ex ) {
            ex.printStackTrace();
        }
    };

    public static XSingleComponentFactory __getComponentFactory( String sImplementationName ) {
        XSingleComponentFactory xFactory = null;

        if ( sImplementationName.equals( m_implementationName ) )
            xFactory = Factory.createComponentFactory(OOoNameRecognizer.class, m_serviceNames);
        return xFactory;
    }

    public static boolean __writeRegistryServiceInfo( XRegistryKey xRegistryKey ) {
        return Factory.writeRegistryServiceInfo(m_implementationName,
                                                m_serviceNames,
                                                xRegistryKey);
    }

    // com.sun.star.lang.XServiceInfo:
    public String getImplementationName() {
         return m_implementationName;
    }

    public boolean supportsService( String sService ) {
        int len = m_serviceNames.length;

        for( int i=0; i < len; i++) {
            if (sService.equals(m_serviceNames[i]))
                return true;
        }
        return false;
    }

    public String[] getSupportedServiceNames() {
        return m_serviceNames;
    }

    // com.sun.star.smarttags.XSmartTagRecognizer:
    public int getSmartTagCount() {
        return 1;
    }
    
    // com.sun.star.smarttags.XSmartTagRecognizer:
    public String getSmartTagName(int nSmartTagIndex) throws com.sun.star.lang.IndexOutOfBoundsException {
        return m_SmartTagType;
    }

    // com.sun.star.smarttags.XSmartTagRecognizer:
    public String getName(com.sun.star.lang.Locale aLocale) {
        return new String("OOo Name Recognizer");
    }
    
    // com.sun.star.smarttags.XSmartTagRecognizer:
    public void recognize(String aText, int nStart, int nLength, com.sun.star.smarttags.SmartTagRecognizerMode eDataType, com.sun.star.lang.Locale aLocale, com.sun.star.text.XTextMarkup xTextMarkup, String aApplicationName, com.sun.star.frame.XController xController, com.sun.star.i18n.XBreakIterator xTokenizer) {
        final int nEndPos = nStart + nLength;

        Boundary aWordBounds = xTokenizer.getWordBoundary( aText, nStart, aLocale, WordType.DICTIONARY_WORD, true );

        while ( aWordBounds.startPos < aWordBounds.endPos && aWordBounds.endPos <= nEndPos ) {
            final String aWord = aText.substring( aWordBounds.startPos, aWordBounds.endPos );

            if ( m_aMap.containsKey( aWord ) ) {
                XStringKeyMap xProps = xTextMarkup.getMarkupInfoContainer();
                try {
                    xProps.insertValue("RealName", m_aMap.get( aWord ));
                } catch (com.sun.star.lang.IllegalArgumentException ex) {
                    ex.printStackTrace();
                } catch (ElementExistException ex) {
                    ex.printStackTrace();
                }
                xTextMarkup.commitTextMarkup( TextMarkupType.SMARTTAG,
                                              m_SmartTagType,
                                              aWordBounds.startPos,
                                              aWordBounds.endPos - aWordBounds.startPos,
                                              xProps );
            }

            aWordBounds = xTokenizer.nextWord( aText, aWordBounds.startPos, aLocale, WordType.DICTIONARY_WORD );
        }
    }
    
    // com.sun.star.lang.XInitialization:
    public void initialize(Object[] aArguments) /*throws com.sun.star.uno.Exception*/ {
    }
    
    // com.sun.star.smarttags.XSmartTagRecognizer:
    public boolean hasPropertyPage(int nSmartTagIndex, com.sun.star.lang.Locale aLocale) throws com.sun.star.lang.IndexOutOfBoundsException {
        return true;
    }

    // com.sun.star.smarttags.XSmartTagRecognizer:
    public void displayPropertyPage(int nSmartTagIndex, com.sun.star.lang.Locale aLocale) throws com.sun.star.lang.IndexOutOfBoundsException {
        try {
            final XMultiComponentFactory xFact = m_xContext.getServiceManager();            
            final Object xObject = xFact.createInstanceWithContext("com.sun.star.awt.DialogProvider2", m_xContext);
            final XDialogProvider2 aDialogProvider = (XDialogProvider2) UnoRuntime.queryInterface(XDialogProvider2.class, xObject);
            m_xDialog = aDialogProvider.createDialogWithHandler(m_aDialogFile, this );
            
            // set current search string:
            final XControlContainer xCC = (XControlContainer)UnoRuntime.queryInterface(XControlContainer.class, m_xDialog );
            final XControl xTextField = xCC.getControl("TextField1");
            final XControlModel xTextFieldModel = xTextField.getModel();
            final XPropertySet xPS = (XPropertySet)UnoRuntime.queryInterface(XPropertySet.class, xTextFieldModel );
            
            String aMapData = m_aMap.toString();
            xPS.setPropertyValue( "Text", aMapData.subSequence(1, aMapData.length() - 1) );
            
            m_xDialog.execute();
        } catch ( com.sun.star.uno.Exception ex ) {
            ex.printStackTrace();
        }
    }

    //
    // CURRENTLY NOT USED
    //

    // com.sun.star.smarttags.XSmartTagRecognizer:
    public String getDescription(com.sun.star.lang.Locale aLocale) {
        return new String("Description");
    }

    // com.sun.star.smarttags.XSmartTagRecognizer:
    public String getSmartTagDownloadURL(int nSmartTagIndex) throws com.sun.star.lang.IndexOutOfBoundsException {
        return new String();
    }
    
    //
    // LOCAL HELPER
    //
    
    public boolean callHandlerMethod(XDialog xDialog, Object aEvent, String aEventString ) throws WrappedTargetException {
        if ( aEventString.equals( "ok_buttonup" ) ) {
            final XControlContainer xCC = (XControlContainer)UnoRuntime.queryInterface(XControlContainer.class, xDialog);
            final XControl xTextField = xCC.getControl("TextField1");
            final XControlModel xTextFieldModel = xTextField.getModel();
            final XPropertySet xPS = (XPropertySet)UnoRuntime.queryInterface(XPropertySet.class, xTextFieldModel );
            
            try {
                m_aMap.clear();
                String aMapData = (String) xPS.getPropertyValue( "Text" );
                FillMapData( aMapData );
                
                // update configuration
                m_xConfPropSet.setPropertyValue( "MapData", aMapData );
                final XChangesBatch xChangesBatch = (XChangesBatch)UnoRuntime.queryInterface(XChangesBatch.class, m_xConfPropSet );
                xChangesBatch.commitChanges();
                
                // this little hack will refresh all smart tag data in the application:
                PropertyValue aPathArgument = new PropertyValue();
                aPathArgument.Name = "nodepath";
                aPathArgument.Value = "/org.openoffice.Office.Common/SmartTags/Writer";
        
                Object[] aArgs = new Object[1];
                aArgs[0] = aPathArgument;

                // try to get read access to configuration:
                Object xObject = null;
                try {
                    xObject = m_xConfProv.createInstanceWithArguments( "com.sun.star.configuration.ConfigurationUpdateAccess", aArgs );
                } catch ( com.sun.star.uno.Exception ex ) {
                    ex.printStackTrace();
                }
        
                if ( xObject == null )
                    return true;
        
                // We need to allow extensions to insert their configuration data in the 'regular' 
                // smart tag configuration node. Then the following code (which is a hack to invalidate
                // all smart tags in the document) is not necessary anymore:
                XPropertySet xConfPropSet = (XPropertySet)UnoRuntime.queryInterface(XPropertySet.class, xObject);
                Boolean aRecognizeSmartTags = (Boolean)xConfPropSet.getPropertyValue( "RecognizeSmartTags" );
                xConfPropSet.setPropertyValue( "RecognizeSmartTags", !aRecognizeSmartTags ); // this invalidates all current smart tags
                final XChangesBatch xChangesBatch2 = (XChangesBatch)UnoRuntime.queryInterface(XChangesBatch.class, xConfPropSet );
                xChangesBatch2.commitChanges();
                xConfPropSet.setPropertyValue( "RecognizeSmartTags", aRecognizeSmartTags );
                xChangesBatch2.commitChanges();
            } catch ( com.sun.star.uno.Exception ex ) {
                ex.printStackTrace();
            }

            m_xDialog.endExecute();
        }

        return true;
    }
    
    private void FillMapData( String aMapData ) {
        final String[] aEntries = aMapData.split(",");
        for ( int i = 0; i < aEntries.length; ++i ) {
            String aEntry = aEntries[i];
            aEntry = aEntry.trim();
            final String[] aEntryArr = aEntry.split("=");
            m_aMap.put(aEntryArr[0], aEntryArr[1]);
        }
    }

    public String[] getSupportedMethodNames()  {
        return new String[2];
    }
}
